import pytest
import unittest
import numpy as np
np.random.seed(1234)
from trompy import Lickcalc

def make_toy_data():
    import numpy as np

    short_ilis = np.random.normal(0.16, 0.05, 100)
    med_ilis = [x for x in np.random.poisson(1.5, 10) if x > 0]
    long_ilis = [x for x in np.random.poisson(30, 3) if x > 0]

    all_ilis = np.concatenate([short_ilis, med_ilis, long_ilis])
    np.random.shuffle(all_ilis)

    timestamps = np.cumsum(all_ilis)

    return timestamps

def test_burstcalc():
    licks = make_toy_data()
    # import matplotlib.pyplot as plt
    # plt.plot(licks)
    # plt.show()

    #testing if ocrrect number of bursts identified
    lc = Lickcalc(licks=licks)
    assert lc.burst_number == 11

    # testing if min_burst_length works
    lc = Lickcalc(licks=licks, min_burst_length=5)
    assert lc.burst_number == 8

def test_runcalc():
    licks = make_toy_data()

    # testing if correct number of runs identified
    lc = Lickcalc(licks=licks)
    assert lc.runs_number == 4

def test_longlicks():
    licks = make_toy_data()
    licklengths = np.random.normal(0.06, 0.01, len(licks))
    licklengths[10] = 3
    offsets = licks + licklengths

    # testing if long lick can be identified
    lc = Lickcalc(licks=licks, offset=offsets)
    assert lc.longlicks[0] == 3.0

if __name__ == "__main__":
    test_burstcalc()
    test_runcalc()
    test_longlicks()
    


# if __name__ == '__main__':
#     unittest.main()

# class TestLickCalc(unittest.TestCase):
#     def setUp(self):
#         self.licks = [0.1, 0.2, 0.3, 0.4, 0.5]
#         self.offset = [0.1, 0.2, 0.3, 0.4, 0.5]
#         self.kwargs = {
#             'longlick_threshold': 0.3,
#             'burst_threshold': 0.5,
#             'min_burst_length': 1,
#             'run_threshold': 10,
#             'min_run_length': 1,
#             'binsize': 60,
#             'hist_density': False,
#             'ignorelongilis': False,
#             'licks': self.licks,
#             'offset': self.offset
#         }

#     def test_init(self):
#         lc = LickCalc(**self.kwargs)
#         self.assertEqual(lc.longlick_threshold, 0.3)
#         self.assertEqual(lc.burst_threshold, 0.5)
#         self.assertEqual(lc.min_burst_length, 1)
#         self.assertEqual(lc.run_threshold, 10)
#         self.assertEqual(lc.min_run_length, 1)
#         self.assertEqual(lc.binsize, 60)
#         self.assertEqual(lc.hist_density, False)
#         self.assertEqual(lc.ignorelongilis, False)
#         np.testing.assert_array_equal(lc.licks, np.array(self.licks))
#         np.testing.assert_array_equal(lc.offset, np.array(self.offset))
#         self.assertIsNone(lc.licklength)
#         self.assertIsNone(lc.longlicks)
#         np.testing.assert_array_equal(lc.ilis, np.array([0.1, 0.1, 0.1, 0.1]))
#         self.assertEqual(lc.total, 5)

#     def test_get_burst_inds(self):
#         lc = LickCalc(**self.kwargs)
#         burst_inds = lc.get_burst_inds()
#         expected_burst_inds = [3, 4]
#         np.testing.assert_array_equal(burst_inds, np.array(expected_burst_inds))

#     # Add more test methods for other functions...




# -*- coding: utf-8 -*-
"""
Created on Fri Apr 17 10:14:34 2020

@author: James Edgar McCutcheon
"""

from pathlib import Path
import openpyxl
import xlrd

def metafilereader(filename, sheetname="metafile", delimiter=","):
    """
    Reads in metafile. If an Excel file is given it uses the sheetname argument
    to specify sheet. Otherwise, text files work better than CSV files.

    Parameters
    ----------
    filename : String
        Path to metafile to be read in and interpreted.
    sheetname : String, optional
        Name of sheet within excel file to be used. Default is "metafile".
    delimiter : String, optional
        Delimiter used to separate values in the metafile. Default is ",".

    Returns
    -------
    tablerows : List of lists
        Rows from metafile.
    header : List
        First row from metafile which provides column names.
    """
    filename = Path(filename)
    extension = filename.suffix.lower()

    if extension == ".xlsx":
        
        wb = openpyxl.load_workbook(filename, data_only=True)
        sh = wb[sheetname]  # or wb.sheet_by_name('name_of_the_sheet_here')
        
        header = [cell.value for cell in sh[1]]
        
        tablerows=[]
        for row in sh.iter_rows(min_row=2):
            tablerows.append([cell.value for cell in row])
        
        wb.close()
    
    elif extension == ".xls":
        with xlrd.open_workbook(filename) as wb:
            sh = wb.sheet_by_name(sheetname)  # or wb.sheet_by_name('name_of_the_sheet_here')
            
            header = sh.row_values(0)
            
            tablerows=[]
            for r in range(1, sh.nrows):
                tablerows.append(sh.row_values(r))    
    
    elif extension == ".csv" or extension == '.txt':
        if extension == ".txt":
            delimiter = "\t"

        f = open(filename, 'r')
        f.seek(0)
        header = f.readlines()[0]
        f.seek(0)
        filerows = f.readlines()[1:]
        f.close()
        
        tablerows = []
        
        for i in filerows:
            tablerows.append(i.split(delimiter))
            
        header = header.split(delimiter)

        tablerows = [[item.strip() for item in sublist] for sublist in tablerows]
        header = [item.strip() for item in header]
    else:
        raise ValueError("File extension not supported. Please use .csv, .txt, .xls, or .xlsx")

    return tablerows, header



# -*- coding: utf-8 -*-
"""
Created on Fri Apr 17 16:09:57 2020

@author: James Edgar McCutcheon
"""

import numpy as np
from scipy.stats import t, ttest_rel
from scipy.special import betainc

def sidakcorr(pval, ncomps=3):
    """ Performs Sidak correction on a given p-value"""
    
    corr_p = 1-((1-pval)**ncomps)   
    return corr_p

def mean_and_sem(data, verbose=False):
    """ Calculates mean and standard error of the mean"""
    
    mean = np.mean(data)
    sem = np.std(data)/(np.sqrt(np.size(data)))
    if verbose:
        print(f"Mean: {mean}, SEM: {sem}")
    return (mean, sem)

def bonferroni_corrected_ttest(data1, data2, comps=3, string_prefix="", verbose=False):
    """ Performs a bonferroni corrected t-test"""
    
    t, p = ttest_rel(data1, data2)
    corr_p = p*comps
    if verbose:
        print("{}t-stat: {:03.1f}, corrected p: {:03.4f}".format(string_prefix, np.abs(t), corr_p))
    return (t, corr_p)

## Functions required for calculations of ANOVA statistics
# sum of squares of treatment / regression
def ssr(data):
    sums = []
    for group in data:
        sums.append(len(group) * (np.mean(group) - np.mean(data))**2)
        
    return np.sum(sums)

# sum of squares error
def sse(data):
    sums = []
    for group in data:
        sums.append(np.sum([(x-np.mean(group))**2 for x in group]))
    
    return np.sum(sums)

def dfr(data):
    return np.shape(data)[0] - 1
    
def dfe(data):
    if np.ndim(data) == 1:
        return np.sum([len(x) for x in data]) - len(data)
    elif np.ndim(data) == 2:
        return np.size(data) - np.shape(data)[0]

def msr(data):
    return ssr(data) / dfr(data)

def mse(data):
    return sse(data) / dfe(data)

def fval(data):
    return msr(data) / mse(data)

def pval(data):
    """
    Finds p-value associated with combination of F and df values.
    
    Uses incomplete betfunction from scipy. Also discussed here:
    https://stackoverflow.com/questions/38113120/calculate-f-distribution-p-values-in-python
    """
    F, df1, df2 = fval(data), dfr(data), dfe(data)
    return 1 - betainc(.5*df1, .5*df2, float(df1)*F/(df1*F+df2))

def lsd(data, groups, alpha=0.025):
    return t.ppf(1-alpha, dfe(data)) * np.sqrt(mse(data) * (1/len(data[groups[0]]) + 1/len(data[groups[1]])))

def lsd_pval(data, groups):
    """
    Returns two-tailed p-value for LSD (least significant difference) posthoc tests.
    
    Args
    data : Numpy array
        Data to be tested, m x n array where first dimension are groups (between) and second dim is subjects (within)
    groups : List or tuple
        Two integers specifying groups to be compared.
        
    Returns
        
    """
    if np.ndim(data) == 1:
        gp1 = data[groups[0]]
        gp2 = data[groups[1]]
    elif np.ndim(data) == 2:
        gp1 = data[groups[0],:]
        gp2 = data[groups[1],:]
    else:
        print("Input data has too many dimensions. Exiting.")
        return
    
    diff = np.abs(np.mean(gp1) - np.mean(gp2))
    
    tval = diff / np.sqrt(mse(data) * (1/len(gp1) + 1/len(gp2)))
    pval = t.sf(np.abs(tval), dfe(data))*2
    
    return pval



from pathlib import Path
import numpy as np
from scipy import stats
import scipy.optimize as opt

class Lickcalc:
    def __init__(self, **kwargs):
        ## Set default parameters

        if "longlick_threshold" in kwargs:
            self.longlick_threshold = kwargs['longlick_threshold']
        else:
            self.longlick_threshold = 0.3

        if "burst_threshold" in kwargs:
            self.burst_threshold = kwargs['burst_threshold']
        else:
            self.burst_threshold = 0.5

        if "min_burst_length" in kwargs:
            self.min_burst_length = kwargs['min_burst_length']
        else:
            self.min_burst_length = 1

        if "run_threshold" in kwargs:
            self.run_threshold = kwargs['run_threshold']
        else:
            self.run_threshold = 10

        if "min_run_length" in kwargs:
            self.min_run_length = kwargs['min_run_length']
        else:
            self.min_run_length = 1

        if "binsize" in kwargs:
            self.binsize = kwargs['binsize']
        else:
            self.binsize = 60

        if "hist_density" in kwargs:
            self.hist_density = kwargs['hist_density']
        else:
            self.hist_density = False     

        self.ignorelongilis = kwargs.get('ignorelongilis', False)
        
        ## Read in and process data
        self.licks = np.array(kwargs.get('licks', None))

        if "offset" in kwargs:
            self.offset = np.array(kwargs['offset'])
            self.licklength = self.get_licklengths()
            if len(self.licklength) > 0:
                self.longlicks = self.get_longlicks()
            else:
                self.longlicks = None
        else:
            self.offset = None
            self.licklength = None
            self.longlicks = None
        
        self.ilis = self.get_ilis()
        self.total = self.get_total_licks()

        self.burst_inds = self.get_burst_inds()
        self.burst_licks = self.get_burst_licks()
        self.burst_start = self.get_burst_start()
        self.burst_end = self.get_burst_end()
        self.burst_lengths = self.get_burst_lengths()

        if self.min_burst_length > 1:
            self.remove_short_bursts()

        self.burst_number = self.get_burst_number()
        self.burst_mean = self.get_burst_mean()
        self.burst_mean_first3 = self.get_burst_mean(number=3)
        self.intraburst_freq = self.get_intraburst_freq()
        self.interburst_intervals = self.get_interburstintervals()
        
        # then lick runs
        self.runs = self.get_runs()
        self.runs_start = self.get_run_start()
        self.runs_inds = self.get_run_inds()
        self.runs_end = self.get_run_end()
        self.runs_licks = self.get_run_licks()
        self.runs_length = self.get_run_lengths()
        if self.min_run_length > 1:
            self.remove_short_runs()
        self.runs_number = len(self.runs)
            
        # then burst probability
        self.burst_prob = self.get_burst_probability()
        self.weibull_params = self.get_weibull_params()

        self.histogram = self.get_histogram()

    def get_licklengths(self):
        onsets = self.licks[:len(self.offset)]
        return self.offset - onsets

    def get_longlicks(self):
        if min(self.licklength) < 0:
            print("One or more offsets precede onsets. Not doing lick length analysis.")
            return None
        else:
            return [x for x in self.licklength if x > self.longlick_threshold]

    def get_ilis(self):
        if self.ignorelongilis:
            return [x for x in np.diff(self.licks) if x < self.burst_threshold]
        else:
            return np.diff(self.licks)

    def get_total_licks(self):
        return len(self.licks)

    def get_burst_inds(self):
        burst_inds = (np.where(np.diff(self.licks) > self.burst_threshold)[0] + 1).tolist()
        if burst_inds[0] != 0:
            burst_inds = [0] + burst_inds
        return burst_inds
    
    def get_burst_licks(self):
        return [int(x) for x in np.diff(self.burst_inds + [self.get_total_licks()])]
    
    def get_burst_start(self):
        return self.licks[self.burst_inds].tolist()
    
    def get_burst_end(self):
        end = self.licks[np.array(self.burst_inds[1:]) - 1]
        return end.tolist() + [self.licks[-1]]
    
    def get_burst_lengths(self):
        return [x - y for x, y in zip(self.burst_end, self.burst_start)]
    
    def remove_short_bursts(self):
        inds_to_keep = [i for i, val in enumerate(self.burst_licks) if val >= self.min_burst_length]
        for burst_var in ['burst_inds', 'burst_licks', 'burst_start', 'burst_end', 'burst_lengths']:
            setattr(self, burst_var, [getattr(self, burst_var)[i] for i in inds_to_keep])

    def get_burst_number(self):
        return len(self.burst_inds)
    
    def get_burst_mean(self, number=None):
        if self.burst_number == 0:
            return None
        if number == None:
            return np.mean(self.burst_licks)
        elif number < self.burst_number:
            return np.mean(self.burst_licks[number])
        else:
            return np.mean(self.burst_licks)

    def get_intraburst_freq(self):
        if self.burst_number == 0 or np.max(self.burst_licks) < 2:
            return None
        else:
            return 1/np.mean([x for x in self.get_ilis() if x < self.burst_threshold])

    def get_interburstintervals(self):
        if self.burst_number == 0:
            return None
        else:
            return np.array(self.burst_start[1:]) - np.array(self.burst_end[:-1])
    
    def get_runs(self):
        split_points = np.where(np.array(self.get_ilis()) > self.run_threshold)[0] + 1
        return np.split(self.licks, split_points)
    
    def get_run_start(self):
        return [run[0] for run in self.runs]
    
    def get_run_inds(self):
        return np.where(np.isin(self.licks, self.runs_start))[0].tolist()
    
    def get_run_end(self):
        return [run[-1] for run in self.runs]
    
    def get_run_licks(self):
        return [len(run) for run in self.runs]
    
    def get_run_lengths(self):
        return [run[-1] - run[0] for run in self.runs]
    
    def remove_short_runs(self):
        inds_to_keep = [i for i, val in enumerate(self.runs_licks) if val >= self.min_run_length]
        for run_var in ['runs', 'runs_start', 'runs_inds', 'runs_end', 'runs_licks', 'runs_length']:
            setattr(self, run_var, [getattr(self, run_var)[i] for i in inds_to_keep])

    def get_burst_probability(self):
        if self.burst_number == 0:
            return None
        else:
            return calculate_burst_prob(self.burst_licks)
    
    def get_weibull_params(self):
        if self.burst_number == 0:
            return None
        else:
            try:
                return fit_weibull(self.burst_prob[0], self.burst_prob[1])
            except:
                return None
    
    def get_histogram(self):
        bins = np.arange(0, len(self.licks), self.binsize)
        return np.histogram(self.licks, bins=bins, density=self.hist_density)[0]
    
def calculate_burst_prob(bursts):
    """
    Calculates cumulative burst probability as in seminal Davis paper
    DOI: 10.1152/ajpregu.1996.270.4.R793

    Parameters
    ----------
    bursts : List
        List with lengths of bursts.

    Returns
    -------
    x : List
        x values (bin sizes).
    y : List
        y values of cumulative burst probability.

    """
    bins = np.arange(min(bursts), max(bursts))
    hist=np.histogram(bursts, bins=bins, density=True)
    cumsum=np.cumsum(hist[0])

    x = hist[1][1:]
    y = [1-val for val in cumsum]
    
    return x, y

def weib_davis(x, alpha, beta):
    '''Weibull function as used in Davis (1998) DOI: 10.1152/ajpregu.1996.270.4.R793'''
    return (np.exp(-(alpha*x)**beta))

def fit_weibull(xdata, ydata):
    '''Fits Weibull function to xdata and ydata and returns fit parameters.'''
    x0=np.array([0.1, 1])
    fit=opt.curve_fit(weib_davis, xdata, ydata, x0)
    alpha=fit[0][0]
    beta=fit[0][1]
    slope, intercept, r_value, p_value, std_err = stats.linregress(ydata, weib_davis(xdata, alpha, beta))
    r_squared=r_value**2
    
    return alpha, beta, r_squared

    # need to test with different files and times when zero licks are given etc
        # and when licks are gtiven but no bursts


# idea to add for some of these calculator functions optional arguments that allow one to specify subsets of data
# to make it easier to for example do calculations of each quarter of a session - by licks, or by time
# or to do calculations for different types of licks (e.g. licks during reward vs licks during non-reward)


if __name__ == '__main__':
    print('Testing functions')
    import trompy as tp
    filename = Path("C:/Users/jmc010/Github/trompy/tests/test_data/03_W.med")

    data = tp.medfilereader(filename, vars_to_extract=["e", "f"], remove_var_header=True)

    lickdata = Lickcalc(licks=data[0], offset=data[1], min_burst_length=3)

    # print(dir(lickdata))


    lickdata = Lickcalc(licks=[2.3,5.6,10.2])


    # lickdata = Lickcalc(licks="hello")



# -*- coding: utf-8 -*-
"""
Created on Fri Apr 17 10:12:10 2020

@author: James Edgar McCutcheon
"""
from pathlib import Path
import numpy as np
import string
import datetime

def medfilereader(filename, vars_to_extract='all', session_to_extract=1, verbose=False, remove_var_header=False, dictionary_output=False, **kwargs):
    """
    Reads in Med Associates file stored as single column and returns variables as lists.

    Parameters
    ----------
    filename : str
        File to be read in.
    vars_to_extract : str or list of str, optional
        Variables to extract from the file. Default is 'all'.
    session_to_extract : int, optional
        Specifies the session to extract from the file. Default is 1.
    verbose : bool, optional
        If True, prints statements with file information. Default is False.
    remove_var_header : bool, optional
        If True, removes the first value in each variable array. Useful when negative numbers are used as markers to signal array start. Default is False.
    dictionary_output : bool, optional
        If True, returns the variables as a dictionary with variable names as keys. Default is False.
    **kwargs : dict, optional
        Additional keyword arguments.

    Returns
    -------
    vars_to_return : list or dict
        Variables extracted from the medfile. If dictionary_output is False, returns a list of lists of numbers (int or float). If dictionary_output is True, returns a dictionary with variable names as keys and lists of numbers as values.
    """
    # Function code goes here
    pass
def medfilereader(filename, vars_to_extract = 'all',
                  session_to_extract = 1,
                  verbose = False,
                  remove_var_header = False,
                  dictionary_output = False,
                  **kwargs):
    
    """Reads in Med Associates file stored as single column and returns variables as lists.
    
    Parameters
    ----------
    filename : str
        File to be read in.
    varsToExtract : str or list of str, optional
        (e.g. ['a', 'b', 'f']), default is 'all'
    sessionToExtract : int, optional
        Can be specified for situations in which more than one session is included in a single file. Deafult is 1.
    verbose : bool, optional
        Prints statements with file information. Default is False.
    remove_var_header : bool, optional
        Removes first value in array, useful when negative numbers are used as markers to signal array start. Default is False.
    
    Returns
    -------
    varsToReturn : list of lists of numbers (int or float)
        Variables extracted from medfile as lists or a list of lists ('all')
    """
    if 'varsToExtract' in kwargs:
        vars_to_extract = kwargs['varsToExtract']

    if 'sessionToExtract' in kwargs:
        vars_to_extract = kwargs['sessionToExtract']

    if vars_to_extract == 'all':
        num_vars_to_extract = np.arange(0,26)
    else:
        num_vars_to_extract = [ord(x.lower())-97 for x in vars_to_extract]
    
    f = open(Path(filename), 'r')
    f.seek(0)
    filerows = f.readlines()[8:]
    datarows = [isnumeric(x) for x in filerows]
    matches = [i for i,x in enumerate(datarows) if x == 0.3]
    if session_to_extract > len(matches):
        print('Session ' + str(session_to_extract) + ' does not exist.')
    if verbose == True:
        print('There are ' + str(len(matches)) + ' sessions in ' + filename)
        print('Analyzing session ' + str(session_to_extract))
    
    varstart = matches[session_to_extract - 1]
    medvars = [[] for n in range(26)]
    
    k = int(varstart + 27)
    for i in range(26):
        medvarsN = int(datarows[varstart + i + 1])
        
        medvars[i] = datarows[k:k + int(medvarsN)]
        k = k + medvarsN
        
    if remove_var_header == True:
        vars_to_return = [medvars[i][1:] for i in num_vars_to_extract]
    else:
        vars_to_return = [medvars[i] for i in num_vars_to_extract]

    if len(vars_to_return) == 1:
        vars_to_return = vars_to_return[0]

    if dictionary_output == True:
        num_vars_to_extract = [chr(x + 97) for x in num_vars_to_extract]
        vars_to_return = {num_vars_to_extract[i]: vars_to_return[i] for i in range(len(vars_to_return))}

    return vars_to_return

def isnumeric(s):
    """Converts strings into numbers (floats)

    Parameters
    ----------
    s : any
    
    Returns
    -------
    x : float
    """
    
    try:
        x = float(s)
        return x
    except ValueError:
        return float('nan')
    
def checknsessions(filename):
    '''Helper function for medfilereader that checks how many sessions in medfile'''
    
    f = open(filename, 'r')
    f.seek(0)
    filerows = f.readlines()[8:]
    datarows = [isnumeric(x) for x in filerows]
    matches = [i for i,x in enumerate(datarows) if x == 0.3]
    return matches

def tstamp_to_tdate(timestamp, fmt):
    '''Converts timestamp in string format into datetime object'''
    
    try:
        return datetime.datetime.strptime(timestamp, fmt)
    except ValueError:
        return

def medfilereader_licks(filename,
                  sessionToExtract = 1,
                  verbose = False,
                  remove_var_header = True):
    
    """Reads in Med Associates file stored as single column and returns variables as lists.
    
    Parameters
    ----------
    filename : str
        File to be read in.
    sessionToExtract : int, optional
        Can be specified for situations in which more than one session is included in a single file. Deafult is 1.
    verbose : bool, optional
        Prints statements with file information. Default is False.
    remove_var_header : bool, optional
        Removes first value in array, useful when negative numbers are used as markers to signal array start. Default is False.
    
    Returns
    -------
    medvars : list of lists of numbers (int or float)
        Variables extracted from medfile as lists or a list of lists ('all')
    """
    
    f = open(filename, 'r')
    f.seek(0)
    filerows = f.readlines()[8:]
    datarows = [isnumeric(x) for x in filerows]
    matches = [i for i,x in enumerate(datarows) if x == 0.3]
    if sessionToExtract > len(matches):
        print('Session ' + str(sessionToExtract) + ' does not exist.')
    if verbose == True:
        print('There are ' + str(len(matches)) + ' sessions in ' + filename)
        print('Analyzing session ' + str(sessionToExtract))
    
    varstart = matches[sessionToExtract - 1]    
    medvars = {}
   
    k = int(varstart + 27)
    for i in range(26):
        medvarsN = int(datarows[varstart + i + 1])
        if medvarsN > 1:
            medvars[string.ascii_uppercase[i]] = datarows[k:k + int(medvarsN)]
        k = k + medvarsN
    
    if remove_var_header == True:
        for val in medvars.values():
            val.pop(0)

    return medvars

if __name__ == '__main__':
    print('Testing functions')
    import trompy as tp
    filename = Path("C:/Users/jmc010/Github/trompy/tests/test_data/!2016-08-12_09h33m.Subject 14")

    data = tp.medfilereader(filename, vars_to_extract=["e", "x"], dictionary_output=True)
    print(len(data))
    print(data.keys())

    for key, val in data.items():
        print(key, len(val))



# %%
"""
Created on Tue Mar 29 2022

@author: jmc010
"""
import pytest
import numpy as np
import trompy as tp
import matplotlib.pyplot as plt

#%matplotlib inline

np.random.seed(222)

def make_data(n, dtype="int"):
    if dtype == "int":
        return np.random.randint(10, size=n)
    else:
        return np.random.random(n)

def make_data_array():
    data_array={}
    data_array["one bar"] = [make_data(9)]
    data_array["5x1, balanced"] = [make_data(9), make_data(9), make_data(9), make_data(9), make_data(9)]
    data_array["5x1, unbalanced"] = [make_data(9), make_data(8), make_data(3), make_data(7), make_data(5)]
    data_array["2x2, balanced"] = [[make_data(9), make_data(9)], [make_data(9), make_data(9)]]
    data_array["2x2, unbalanced"] = [[make_data(9), make_data(9)], [make_data(8), make_data(8)]]
    
    tmplist = list(make_data(9))
    tmplist2 = list(make_data(7))

    data_array["2x4, balanced"] = [[tmplist, tmplist, tmplist, tmplist], [tmplist, tmplist, tmplist, tmplist]]
    data_array["2x4, unbalanced"] = [[tmplist, tmplist, tmplist, tmplist], [tmplist2, tmplist2, tmplist2, tmplist2]]

    return data_array


def test_alpha():
    output = tp.barscatter([[1, 2, 3, 4], [5, 6, 7, 8]], scatteralpha=0.2, paired=True)


def check_colors(data_in, n_colors):
    colors = [(1.0, 0.0, 0.0, 1), (0.0, 1.0, 0.0, 1), (0.0, 0.0, 1.0, 1),
              (0.0, 1.0, 1.0, 1), (1.0, 0.0, 1.0, 1)]*2
    colors_in = colors[:n_colors]
    _, _, bars, _ = tp.barscatter(data_in, barfacecolor_option="individual", barfacecolor = colors_in)

    colors_out = []
    for bar in bars:
        colors_out.append(bar.get_children()[0].get_facecolor())

    return colors_in == colors_out

def test_many_colors():
    data_array = make_data_array()

    N_COLORS=[1, 5, 5, 4, 4, 8, 8]
    for key, n_colors in zip(data_array, N_COLORS):
        assert check_colors(data_array[key], n_colors)

def test_kwargs():
    data_in = [make_data(10)]
    tp.barscatter(data_in, errorbars=True, error_bars=False)

def test_improper_structures():
    
    # # unbalanced, grouped
    data_in = [[np.random.randint(10, size=3), np.random.randint(10, size=4), np.random.randint(10, size=2)], \
         [np.random.randint(10, size=5), np.random.randint(10, size=6)]]
    with pytest.raises(ValueError):
        tp.barscatter(data_in)

    data_in = data_in = [make_data(9), make_data(8), "hey there", make_data(7), make_data(5)]
    with pytest.raises(TypeError):
        tp.barscatter(data_in)

def test_spaced():
    data_in = [make_data(3)]
    tp.barscatter(data_in, spaced=True)

def test_different_working_structures():
    data_array = make_data_array()

    tp.barscatter(data_array["one bar"])
    tp.barscatter(data_array["5x1, unbalanced"])
    tp.barscatter(data_array["5x1, unbalanced"])
    tp.barscatter(data_array["2x2, balanced"])
    tp.barscatter(data_array["2x2, balanced"], paired=True)
    tp.barscatter(data_array["2x2, unbalanced"])
    tp.barscatter(data_array["2x2, unbalanced"], paired=True)
    tp.barscatter(data_array["2x4, balanced"])
    tp.barscatter(data_array["2x4, balanced"], paired=True)
    tp.barscatter(data_array["2x4, unbalanced"])
    tp.barscatter(data_array["2x4, unbalanced"], paired=True)

if __name__ == "__main__":
    # test_alpha()
    # test_improper_structures()
    # test_different_working_structures()
    # test_many_colors()
    # test_kwargs()
    # test_alpha()
    test_spaced()
    



# %%



# %%
"""
Created on Wed Mar 23 13:57:05 2022

@author: jmc010
"""
from audioop import add
import pytest
import numpy as np
import trompy as tp
import matplotlib.pyplot as plt

# %matplotlib inline

np.random.seed(222)

def create_gcamp_kernel(t, tau=2.6):
    K = (1/tau)*(np.exp(-t/tau))
    return K

def create_data_stream(n_samples, fs, kernel_process, events, add_noise=False):

    # to create gcamp kernel
    x=np.arange(0, 30, 1/fs)
    kernel=[kernel_process(t) for t in x]

    # converts events into sample numbers
    events_in_samples = [int(event*fs) for event in events]

    # creates stream
    stream = np.zeros(n_samples- len(kernel) + 1 )
    stream[events_in_samples] = [np.abs(np.random.normal(scale=20))+1 for e in range(len(events))]

    simulated_data_stream = np.convolve(stream, kernel, "full")

    if add_noise:
        simulated_data_stream = simulated_data_stream + np.random.normal(0,0.1,n_samples)

    return simulated_data_stream

def add_noise(data_stream, magnitude=0.5):
    return data_stream + np.random.normal(0,magnitude,len(data_stream))

def add_drift(data_stream, freq=0.0001, magnitude=10):
    x = np.arange(len(data_stream))
    low_freq_signal = magnitude * np.sin(x/(1/freq))
    
    return data_stream + low_freq_signal

def double_exponential(t, a1, b1, a2, b2):
    return a1 * np.exp(-t/b1) + a2 * np.exp(-t/b2)

def test_length():
    n_samples = 600000
    fs = 1017.324
    n_events = 150

    data1 = create_data_stream(n_samples, fs, create_gcamp_kernel, [])
    data2 = create_data_stream(n_samples, fs, create_gcamp_kernel, [])

    processed = tp.processdata(data1, data2)

    assert len(processed) == n_samples

def test_df():
    n_samples = 600000
    fs = 1017.324

    events = np.arange(6000, 54000, 2400) / 100

    simulated_gcamp = create_data_stream(n_samples, fs, create_gcamp_kernel, events)

    noise = add_noise(np.zeros(n_samples))

    t=np.arange(n_samples)
    simulated_405 = double_exponential(t, 400, 2e6, 100, 1e5) + noise
    simulated_gcamp_plus_noise = simulated_gcamp + double_exponential(t, 300, 2e6, 75, 1e5) + noise

    processed_lerner = tp.processdata(simulated_gcamp_plus_noise, simulated_405, method="lerner", normalize=False)
    # processed_konanur = tp.processdata(simulated_gcamp_plus_noise, simulated_405, method="konanur", normalize=False)

    f, ax = plt.subplots(nrows=4)
    ax[0].plot(simulated_405)
    ax[1].plot(simulated_gcamp_plus_noise)
    ax[2].plot(processed_lerner)
    # ax[3].plot(processed_konanur)

    event_indices = [int(event*fs) for event in events]

    assert all(event > 0 for event in processed_lerner[event_indices])
    assert processed_lerner[61039] > 4
    assert processed_lerner[524939] > 4

    # TODO improve konanur function and checking, maybe using real data to examine freqs and fits
    # TODO check with reduced sampling frequencies

def future_test_process_data_with_drift():
    n_samples = 600000
    fs = 1017.324
    n_events = 150

    events = np.random.randint(6000, high=54000, size=n_events) / 100

    simulated_gcamp = create_data_stream(n_samples, fs, create_gcamp_kernel, events)
    
    noise = add_noise(np.zeros(n_samples))

    t=np.arange(n_samples)
    simulated_405 = double_exponential(t, 400, 2e6, 100, 1e5) + noise
    simulated_gcamp_plus_noise = simulated_gcamp + double_exponential(t, 300, 2e6, 75, 1e5) + noise

    f, ax = plt.subplots(nrows=3)
    ax[0].plot(simulated_405)
    ax[1].plot(simulated_gcamp_plus_noise)

    processed_signal = tp.processdata(simulated_gcamp_plus_noise, simulated_405, method="lerner", normalize=False)

    ax[2].plot(processed_signal)
    simulated_405_plus_drift = add_drift(simulated_405)
    simulated_gcamp_plus_drift = add_drift(simulated_gcamp_plus_noise)

    processed_signal2 = tp.processdata(simulated_405_plus_drift, simulated_gcamp_plus_drift, method="lerner", normalize=False)
    
    f, ax = plt.subplots(nrows=3)
    ax[0].plot(simulated_405_plus_drift)
    ax[1].plot(simulated_gcamp_plus_drift)
    ax[2].plot(processed_signal2)

if __name__ == "__main__":
    # test_process_data()
    test_length()
    test_df()
    
# %%

# %%



import pytest
from pathlib import Path
import trompy as tp

def specify_filenames():
    string_csv_file = "./test_data/test_metafile.csv"
    string_txt_file = "./test_data/test_metafile.txt"
    string_xls_file = "./test_data/test_metafile.xls"
    string_xlsx_file = "./test_data/test_metafile.xlsx"

    files = [string_xlsx_file, string_xls_file, string_csv_file, string_txt_file]

    return files

def read_in_string_file():

    files = specify_filenames()
    for file in files:
        tablerows, header = tp.metafilereader(file)
        print(header)
        assert(len(header) == 3)

def read_in_pathlib_file():

    files = specify_filenames()
    for file in files:
        file = Path(file)
        tablerows, header = tp.metafilereader(file)
        assert(len(header) == 3)

# def test_delimiters():

#     for file in files:
#         tablerows, header = tp.metafilereader(file, delimiter=",")


if __name__ == "__main__":


    read_in_string_file()
    read_in_pathlib_file()



# -*- coding: utf-8 -*-
# %%
"""
Created on Wed Mar 23 13:57:05 2022

@author: jmc010
"""
from email.mime import base
import pytest
import numpy as np
import trompy as tp
import matplotlib.pyplot as plt

# %matplotlib inline

np.random.seed(222)

def test_output():
    data = np.random.random(10000)
    events = np.random.randint(1000, high=90000, size=5) / 10
    output, pps = tp.snipper(data, events)
    assert np.shape(output) == (5, 30)

    events[2] = np.nan
    events[3] = np.inf
    output, pps = tp.snipper(data, events)
    assert np.shape(output) == (3, 30) #checks that nans and infs removed

def create_gcamp_kernel(t, tau=2.6):
    K = (1/tau)*(np.exp(-t/tau))
    return K

def create_data_stream(n_samples, fs, kernel_process, events):

    # to create gcamp kernel
    x=np.arange(0, 30, 1/fs)
    kernel=[kernel_process(t) for t in x]

    # converts events into sample numbers
    events_in_samples = [int(event*fs) for event in events]

    # creates stream
    stream = np.zeros(n_samples- len(kernel) + 1 )
    stream[events_in_samples] = [np.abs(np.random.normal(scale=20)) for e in range(len(events))]

    simulated_data_stream = np.convolve(stream, kernel, "full")

    return simulated_data_stream

def create_stream_with_events(n_samples=600000, fs=1017.324, kernel_process=create_gcamp_kernel, n_events=3):

    events = np.random.randint(6000, high=54000, size=n_events) / 100
    simulated_gcamp = create_data_stream(n_samples, fs, create_gcamp_kernel, events)

    return simulated_gcamp, events, fs

def test_peaks():
    np.random.seed(222)
    simulated_gcamp, events, fs = create_stream_with_events()
    output, _ = tp.snipper(simulated_gcamp, events, fs=fs, adjustBaseline = False)

    for trace in output:
        assert np.argmax(trace) == 10172

def test_adjust_baseline():
    np.random.seed(222)
    simulated_gcamp, events, fs = create_stream_with_events()

    x = np.arange(len(simulated_gcamp))
    low_freq_signal = np.sin(x/100000)
    simulated_gcamp_with_drift = simulated_gcamp + low_freq_signal
    
    output, _ = tp.snipper(simulated_gcamp_with_drift, events, fs=fs, adjust_baseline = True)

    for trace in output:
        np.testing.assert_allclose(np.mean(trace[:10000]), 0.0, atol=0.1)
        assert trace[10173] > 0

def test_bins():
    np.random.seed(222)
    simulated_gcamp, events, fs = create_stream_with_events()

    for n_bins in [30]:
        output, _ = tp.snipper(simulated_gcamp, events, fs=fs, adjust_baseline = True, bins=n_bins)
        scaling_factor = n_bins/30

        assert np.shape(output)[1] == n_bins

        for trace in output:
            print(trace[int(scaling_factor*11)])
            np.testing.assert_allclose(np.mean(trace[:int(scaling_factor*10)]), 0.0, atol=0.1)
            assert trace[int(scaling_factor*11)] > 2

def test_trial_length():
    np.random.seed(222)
    simulated_gcamp, events, fs = create_stream_with_events()

    f, ax = plt.subplots()
    for pre in [1, 5, 10]:
        for length in [12, 15, 20, 30]:
            output, _ = tp.snipper(simulated_gcamp, events, fs=fs, adjust_baseline = True,
                                    baseline_length=pre,
                                    trial_length=length,
                                    bins=length*10)
            baseline = np.mean(output[:,:pre*10])
            np.testing.assert_allclose(baseline, 0.0, atol=0.1)
            assert np.mean(output[:,(pre+1)*10]) > 4

def test_no_events():
    np.random.seed(222)
    data = np.random.random(10000)
    events=[]

    with pytest.raises(Exception):
        _, _ = tp.snipper(data, events, fs=10)

def test_overlap():
    np.random.seed(222)

    for fs in [10, 100, 1017.234]:
        data = np.random.random(int(120*fs))
        events=[1, 60, 119]

        output, _ = tp.snipper(data, events, fs=fs)
        assert len(output) == 1

# TODO: check with varied fs

if __name__ == "__main__":
    test_output()
    test_peaks()
    test_adjust_baseline() 
    test_bins()
    test_trial_length()
    test_no_events()
    test_overlap()


# %%



# -*- coding: utf-8 -*-
"""
Created on Fri Apr 17 13:40:05 2020

@author: James Edgar McCutcheon
"""
from pathlib import Path

def lickCalc(licks, offset = [], burstThreshold = 0.5, runThreshold = 10,
             ignorelongilis=True, longlickThreshold=0.3, minburstlength=1,
             minrunlength=1,
             binsize=60, histDensity = False):
    """
    Calcuates various parameters for a train of licking data including bursting 
    parameters and returns as a dictionary. Legacy function that returns a dictionary.

    Parameters
    ----------
    licks : List or 1D array
        Timestamps of licks (onsets). Expects in seconds. If given in ms then other
        parameters should be changed, e.g. `burstThreshold`.
    offset : List or 1D array, optional
        Timestamps of lick offsets. The default is [].
    burstThreshold : Float, optional
        Interlick threshold (in seconds) for defining bursts. The default is 0.5.
    runThreshold : Float or Int, optional
        Number of seconds separating runs of licks. The default is 10.
    ignorelongilis : Boolean, optional
        Removes ILIs > `burstThreshold` from calcuations. Useful for calculating
        intraburst frequency. The default is True.
    longlickThreshold : Float, optional
        Threshold (in seconds) for long licks. The default is 0.3.
    minburstlength : Int, optional
        Minimum number of licks to be considered a burst. Common values are either 1 or 3. The default is 1.
    binsize : Int or Float, optional
        Size of bins for constructing histogram. The default is 60.
    histDensity : Boolean, optional
        Converts histogram into a density plot rather than absolute. The default is False.

    Returns
    -------
    lickdata, a dictionary with the following keys
    'licklength' : List of floats
        Lick lengths (empty if offset times not given)
    'longlicks' : List of floats
        Licks that are greater than `longlickThreshold`
    'licks' : List of floats
        Lick onset times
    'ilis' : List of floats
        Interlick intervals
    'freq' : Float
        Mean frequency (in Hz) of intraburst licking
    'total' : Int
        Number of licks
    'bStart' : List of floats
        Timestamps of licks that start a burst
    'bInd' : List of ints
        Indices of licks that start a burst
    'bEnd' : List of floats
        Timestamps of licks that end a burst
    'bLicks' : List of ints
        Numbers of licks in each burst
    'bTime' : Array of floats
        Duration (in seconds) of each burst
    'bMean' : Float
        Mean number of licks in all bursts
    'bMean-first3' : Float
        Mean number of licks in first three bursts
    'IBIs' : List of floats
        Interburst intervals
    'rStart' : List of floats
        Timestamps for licks that start a run. Runs designated by `runThreshold`
    'rInd' : List of ints
        Indices of licks that start a run
    'rEnd' : List of ints
        Timestamps of licks that end a run
    'rLicks' : List of ints
        Number of licks per run
    'rTime' : Array of floats
        Duration (in seconds) of each run
    'rNum' : Int
        Number of runs
    'weib_alpha' : Float
        Alpha parameter from fitted Weibull function
    'weib_beta' : Float
        Beta parameter from fitted Weibull function
    'weib_rsq' : Float
        Rsquared value from fitted Weibull function
    'burstprob' : List
        xdata and ydata of cumulative burst probability
    'hist' : List
        Histogram of licks over time
    
    Notes
    ----------
    For more information on appropriate thresholds (e.g. numbers of licks or interlick intervals) see Naneix et al (2019) for more info.
        https://doi.org/10.1016/j.neuroscience.2019.10.036
    
    For more information on the Weibull function used to model burst probability see Davis (1998) DOI: 10.1152/ajpregu.1996.270.4.R793
    """

    lickdata = Lickcalc(licks=licks,
                        offset=offset,
                        burst_threshold=burstThreshold,
                        longlick_threshold=longlickThreshold,
                        min_burst_length=minburstlength,
                        ignorelongilis=ignorelongilis,
                        run_threshold=runThreshold,
                        min_run_length=minrunlength,
                        binsize=binsize,
                        hist_density=histDensity)
    
    if lickdata.weibull_params is None:
        lickdata.weibull_params = [None, None, None]
    
    return {'licklength' : lickdata.licklength,
            'longlicks' : lickdata.longlicks,
            'licks' : lickdata.licks,
            'ilis' : lickdata.ilis,
            'freq' : lickdata.intraburst_freq,
            'total' : lickdata.total,
            'bStart' : lickdata.burst_start,
            'bInd' : lickdata.burst_inds,
            'bEnd' : lickdata.burst_end,
            'bLicks' : lickdata.burst_licks,
            'bNum': lickdata.burst_number,
            'bTime' : lickdata.burst_lengths,
            'bMean' : lickdata.burst_mean,
            'bMean-first3' : lickdata.burst_mean_first3,
            'IBIs' : lickdata.interburst_intervals,
            'rStart' : lickdata.runs_start,
            'rInd' : lickdata.runs_inds,
            'rEnd' : lickdata.runs_end,
            'rLicks' : lickdata.runs_licks,
            'rTime' : lickdata.runs_length,
            'rNum' : lickdata.runs_number,
            'weib_alpha' : lickdata.weibull_params[0],
            'weib_beta' : lickdata.weibull_params[1],
            'weib_rsq' : lickdata.weibull_params[2],
            'burstprob' : lickdata.burst_prob,
            'hist' : lickdata.histogram
            }

if __name__ == '__main__':
    print('Testing functions')
    import trompy as tp
    filename = Path("C:/Users/jmc010/Github/trompy/tests/test_data/03_W.med")

    data = tp.medfilereader(filename, vars_to_extract=["e", "f"], remove_var_header=True)

    lickdata = lickCalc(data[0], offset=data[1], minburstlength=10)

    # print(lickdata['freq'])



